==> 0.1.2%2B1.tar.gz: ios/Classes/QUQRCodeScannerViewController.swift <==
//
//  QUQRCodeScannerViewController.swift
//  qu_qr_code_scanner
//
//  Created by edy on 2021/7/15.
//

import UIKit
import AVFoundation


class QUQRCodeScannerViewController: UIViewController, AVCaptureMetadataOutputObjectsDelegate, UIAlertViewDelegate {
    
    var scanRectView: UIView!
    var device: AVCaptureDevice!
    var input: AVCaptureDeviceInput!
    var output: AVCaptureMetadataOutput!
    var session: AVCaptureSession!
    var preview: AVCaptureVideoPreviewLayer!
    
    var gradientLayer = CAGradientLayer()
    var scanRect: CGRect!
    var choseImgBtn = UIButton()
    let maxWindowSize = max(UIScreen.main.bounds.size.width, UIScreen.main.bounds.size.height)
    let minWindowSize = min(UIScreen.main.bounds.size.width, UIScreen.main.bounds.size.height)
    lazy var scanSize = CGSize(width: minWindowSize * 3 / 4, height: minWindowSize * 3 / 4)
    
    var resultBlock: ((_ result: String) ->Void)?
    
    var qrString = ""
    var isEnLarge = true
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        initView()
        initGradientLayer()
        setScanner()
        
        orientationDidChangeNotification()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        startAnimation()
    }
    
    //摄像头捕获
    func metadataOutput(_ output: AVCaptureMetadataOutput,
                        didOutput metadataObjects: [AVMetadataObject],
                        from connection: AVCaptureConnection) {
        var stringValue: String?
        if metadataObjects.count > 0 {
            let metadataObject = metadataObjects[0] as! AVMetadataMachineReadableCodeObject
            stringValue = metadataObject.stringValue
            
            if stringValue != nil {
                print("扫描到的二维码：", stringValue!)
                qrString = stringValue!
                if let b = self.resultBlock {
                    b(qrString)
                }
                self.session.stopRunning()
            }
        }
        self.session.stopRunning()
        self.dismiss(animated: true) {
            
        }
        
    }
    
    private func initGradientLayer() {
        let colors = [UIColor.green.withAlphaComponent(0).cgColor, UIColor.green.withAlphaComponent(0.4).cgColor, UIColor.green.withAlphaComponent(0.8).cgColor]
        gradientLayer.colors = colors
        gradientLayer.frame =  CGRect(x: 5, y: 0, width: scanSize.width - 10, height: 40)
        gradientLayer.locations = [0, 0.6, 1]
    }
    
    func startAnimation() {
        let animation = CABasicAnimation(keyPath: "transform.translation.y")
        animation.fromValue = 0
        animation.toValue = scanSize.height - 40
        animation.duration = 2
        animation.repeatCount = 99999
        let animationOpacity = CABasicAnimation(keyPath: "opacity")
        animationOpacity.fromValue = 0.8
        animationOpacity.toValue = 0.3
        animationOpacity.duration = 2
        animationOpacity.timingFunction = CAMediaTimingFunction(name: CAMediaTimingFunctionName.easeInEaseOut)
        animationOpacity.repeatCount = 99999
        gradientLayer.add(animation, forKey: nil)
        gradientLayer.add(animationOpacity, forKey: nil)
    }
    
    func initView() {
        let backImg = QUImagesHelper.imageFor(name: "back")
        let backBtn = UIBarButtonItem(image: backImg, style: .plain, target: self, action: #selector(backAction))
        backBtn.tintColor = UIColor.white
        navigationItem.setLeftBarButton(backBtn, animated: false)
        
        let img = QUImagesHelper.imageFor(name: "photo")
        choseImgBtn.setImage(img, for: .normal)
        choseImgBtn.addTarget(self, action: #selector(chooseImg), for: .touchUpInside)
        self.view.addSubview(choseImgBtn)
        
        self.view.isUserInteractionEnabled = true
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(tapGesture))
        tapGesture.numberOfTapsRequired = 2
        self.view.addGestureRecognizer(tapGesture)
        
    }
    
    @objc func tapGesture(){
        
        CATransaction.begin()
        CATransaction.setAnimationDuration(0.5)
        if isEnLarge {
            self.preview.transform = CATransform3DMakeScale(3, 3, 3)
        }else{
            self.preview.transform = CATransform3DMakeScale(1, 1, 1)
        }
        CATransaction.commit()
        isEnLarge = !isEnLarge
     
    }
    

    @objc
    private func backAction() {
        navigationController?.dismiss(animated: true, completion: nil)
    }
    
    @objc  func chooseImg() {
        let pick = UIImagePickerController()
        pick.delegate = self
        pick.sourceType = .photoLibrary
        pick.modalPresentationStyle = .fullScreen
        self.present(pick, animated: true, completion: nil)
        
    }
    
    private func orientationDidChangeNotification() {
        let name = UIDevice.orientationDidChangeNotification
        NotificationCenter.default.addObserver(self, selector: #selector(orientationDidChangeHandle), name: name, object: nil)
    }
    
    @objc
    private func orientationDidChangeHandle() {
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
            self.updateScannerRect()
        }
    }
    
    private func setScanner() {
        do {
            // Do any additional setup after loading the view.
            self.device = AVCaptureDevice.default(for: AVMediaType.video)
            
            self.input = try AVCaptureDeviceInput(device: device)
            
            self.output = AVCaptureMetadataOutput()
            output.setMetadataObjectsDelegate(self, queue: DispatchQueue.main)
            
            self.session = AVCaptureSession()
            self.session.addInput(self.input)
            self.session.addOutput(self.output)
            
            self.output.metadataObjectTypes = [AVMetadataObject.ObjectType.qr]
            
            self.preview = AVCaptureVideoPreviewLayer(session: self.session)
            self.preview.videoGravity = AVLayerVideoGravity.resizeAspectFill
            self.view.layer.insertSublayer(self.preview, at: 0)
            
            //添加中间的探测区域绿框
            self.scanRectView = UIView();
            self.view.addSubview( self.scanRectView)
            self.scanRectView.layer.addSublayer(self.gradientLayer)
            
            self.scanRectView.layer.borderColor = UIColor.green.cgColor
            self.scanRectView.layer.borderWidth = 1;
            //开始捕获
            self.session.startRunning()
            
        } catch _ {
            //打印错误消息
            let alertController = UIAlertController(title: "提醒",
                                                    message: "请在iPhone的\"设置-隐私-相机\"选项中,允许本程序访问您的相机",
                                                    preferredStyle: .alert)
            let cancelAction = UIAlertAction(title: "确定", style: .cancel, handler: nil)
            alertController.addAction(cancelAction)
            self.present(alertController, animated: true, completion: nil)
        }
        updateScannerRect()
    }
    
    private func updateScannerRect() {
        startAnimation()
        let viewSize = view.frame.size
        self.preview.frame = self.view.bounds
        choseImgBtn.frame  = CGRect(x: viewSize.width - 80, y: viewSize.height - 100, width: 60, height: 60)
        
        if maxWindowSize < 500 {
            self.session.sessionPreset = AVCaptureSession.Preset.vga640x480
        }else{
            self.session.sessionPreset = AVCaptureSession.Preset.high
        }
        
        //计算中间可探测区域
        scanRect = CGRect(x: (viewSize.width - scanSize.width) / 2,
                          y: (viewSize.height - scanSize.height) / 2,
                          width: scanSize.width, height: scanSize.height)
        //计算rectOfInterest 注意x,y交换位置
        scanRect = CGRect(x: scanRect.origin.y / viewSize.height,
                          y: scanRect.origin.x / viewSize.width,
                          width: scanRect.size.height / viewSize.height,
                          height: scanRect.size.width / viewSize.width);
        //设置可探测区域
        self.output.rectOfInterest = scanRect
        
        self.scanRectView.frame = CGRect(x: 0, y: 0, width: scanSize.width,
                                         height: scanSize.height);
        self.scanRectView.center = CGPoint(x: UIScreen.main.bounds.midX,
                                           y: UIScreen.main.bounds.midY)
        let orientation = UIDevice.current.orientation
        if let connection = preview.connection {
            if connection.isVideoOrientationSupported,
               let videoOrientation = AVCaptureVideoOrientation(rawValue: orientation.rawValue) {
                connection.videoOrientation = videoOrientation
            }
        }
    }
}


extension QUQRCodeScannerViewController: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    
    func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey : Any]) {
        self.dismiss(animated: true, completion: nil)
        var image: UIImage?
        
        image = info[.originalImage] as? UIImage
        
        let ciImage = CIImage(image:image!)!
        
        let context = CIContext(options: nil)
        let detector = CIDetector(ofType: CIDetectorTypeQRCode, context: context, options: [CIDetectorAccuracy:CIDetectorAccuracyHigh])
        
        let features = detector?.features(in: ciImage)
        for feature in features as! [CIQRCodeFeature] {
            qrString = feature.messageString ?? ""
            print("图片读取到的二维码",feature.messageString ?? "")
            if let b = resultBlock {
                b(qrString)
            }
            self.dismiss(animated: true) {
                
            }
        }
    }
    
}

==> 0.1.3%2B1.tar.gz: ios/Classes/QUQRCodeScannerViewController.swift <==
//
//  QUQRCodeScannerViewController.swift
//  qu_qr_code_scanner
//
//  Created by edy on 2021/7/15.
//

import AVFoundation
import UIKit

class QUQRCodeScannerViewController: UIViewController, AVCaptureMetadataOutputObjectsDelegate, UIAlertViewDelegate {
    var scanRectView = UIView()
    var device: AVCaptureDevice!
    var input: AVCaptureDeviceInput!
    var output: AVCaptureMetadataOutput!
    var session: AVCaptureSession!
    var preview: AVCaptureVideoPreviewLayer!

    var scanImg = UIImageView(image: QUImagesHelper.imageFor(name: "scan"))
    var choseImgBtn = UIButton()
    let maxWindowSize = max(UIScreen.main.bounds.size.width, UIScreen.main.bounds.size.height)
    let minWindowSize = min(UIScreen.main.bounds.size.width, UIScreen.main.bounds.size.height)
    lazy var scanSize = CGSize(width: minWindowSize * 3 / 4, height: minWindowSize * 3 / 4)

    var resultBlock: ((_ result: String) -> Void)?

    var qrString = ""
    var isEnLarge = true

    override func viewDidLoad() {
        super.viewDidLoad()

        initView()
        setScanner()

        orientationDidChangeNotification()
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        startAnimation()
    }

    // 摄像头捕获
    func metadataOutput(_: AVCaptureMetadataOutput,
                        didOutput metadataObjects: [AVMetadataObject],
                        from _: AVCaptureConnection)
    {
        var stringValue: String?
        if metadataObjects.count > 0 {
            let metadataObject = metadataObjects[0] as! AVMetadataMachineReadableCodeObject
            stringValue = metadataObject.stringValue

            if stringValue != nil {
                print("扫描到的二维码：", stringValue!)
                qrString = stringValue!
                if let b = resultBlock {
                    b(qrString)
                }
                session.stopRunning()
            }
        }
        session.stopRunning()
        dismiss(animated: true) {}
    }

    func startAnimation() {
        let animation = CABasicAnimation(keyPath: "transform.translation.y")
        animation.fromValue = 0
        animation.toValue = scanSize.height - 30
        animation.duration = 2
        animation.repeatCount = 99999
        scanImg.layer.add(animation, forKey: nil)
    }

    func initView() {
        let backImg = QUImagesHelper.imageFor(name: "back")
        let backBtn = UIBarButtonItem(image: backImg, style: .plain, target: self, action: #selector(backAction))
        backBtn.tintColor = UIColor.white
        navigationItem.setLeftBarButton(backBtn, animated: false)

        let img = QUImagesHelper.imageFor(name: "photo")
        choseImgBtn.setImage(img, for: .normal)
        choseImgBtn.addTarget(self, action: #selector(chooseImg), for: .touchUpInside)
        view.addSubview(choseImgBtn)

        view.isUserInteractionEnabled = true
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(tapGesture))
        tapGesture.numberOfTapsRequired = 2
        view.addGestureRecognizer(tapGesture)
    }

    @objc func tapGesture() {
        CATransaction.begin()
        CATransaction.setAnimationDuration(0.5)
        if isEnLarge {
            preview.transform = CATransform3DMakeScale(3, 3, 3)
        } else {
            preview.transform = CATransform3DMakeScale(1, 1, 1)
        }
        CATransaction.commit()
        isEnLarge = !isEnLarge
    }

    @objc
    private func backAction() {
        navigationController?.dismiss(animated: true, completion: nil)
    }

    @objc func chooseImg() {
        let pick = UIImagePickerController()
        pick.delegate = self
        pick.sourceType = .photoLibrary
        pick.modalPresentationStyle = .fullScreen
        present(pick, animated: true, completion: nil)
    }

    private func orientationDidChangeNotification() {
        let name = UIDevice.orientationDidChangeNotification
        NotificationCenter.default.addObserver(self, selector: #selector(orientationDidChangeHandle), name: name, object: nil)
    }

    @objc
    private func orientationDidChangeHandle() {
        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
            self.updateScannerRect()
        }
    }

    private func setScanner() {
        do {
            // Do any additional setup after loading the view.
            device = AVCaptureDevice.default(for: AVMediaType.video)

            input = try AVCaptureDeviceInput(device: device)

            output = AVCaptureMetadataOutput()
            output.setMetadataObjectsDelegate(self, queue: DispatchQueue.main)

            session = AVCaptureSession()
            session.addInput(input)
            session.addOutput(output)

            output.metadataObjectTypes = [AVMetadataObject.ObjectType.qr]

            preview = AVCaptureVideoPreviewLayer(session: session)
            preview.videoGravity = AVLayerVideoGravity.resizeAspectFill
            view.layer.insertSublayer(preview, at: 0)

            // 添加中间的扫描特效活动范围试图
            view.addSubview(scanRectView)
            scanRectView.addSubview(scanImg)
            // 开始捕获
            session.startRunning()

        } catch _ {
            // 打印错误消息
            let alertController = UIAlertController(title: "提醒",
                                                    message: "请在iPhone的\"设置-隐私-相机\"选项中,允许本程序访问您的相机",
                                                    preferredStyle: .alert)
            let cancelAction = UIAlertAction(title: "确定", style: .cancel, handler: nil)
            alertController.addAction(cancelAction)
            present(alertController, animated: true, completion: nil)
        }
        updateScannerRect()
    }

    private func updateScannerRect() {
        startAnimation()
        let viewSize = view.frame.size
        preview.frame = view.bounds
        choseImgBtn.frame = CGRect(x: viewSize.width - 80, y: viewSize.height - 100, width: 60, height: 60)

        if maxWindowSize < 500 {
            session.sessionPreset = AVCaptureSession.Preset.vga640x480
        } else {
            session.sessionPreset = AVCaptureSession.Preset.high
        }

        scanRectView.frame = CGRect(x: 0, y: 0, width: minWindowSize,
                                    height: minWindowSize)
        scanRectView.center = CGPoint(x: UIScreen.main.bounds.midX,
                                      y: UIScreen.main.bounds.midY)
        scanImg.frame = CGRect(x: 0, y: 0, width: minWindowSize, height: 30)
        scanImg.contentMode = .scaleAspectFit

        let orientation = UIDevice.current.orientation
        if let connection = preview.connection {
            if connection.isVideoOrientationSupported,
               let videoOrientation = AVCaptureVideoOrientation(rawValue: orientation.rawValue)
            {
                connection.videoOrientation = videoOrientation
            }
        }
    }
}

extension QUQRCodeScannerViewController: UIImagePickerControllerDelegate, UINavigationControllerDelegate {
    func imagePickerController(_: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey: Any]) {
        dismiss(animated: true, completion: nil)
        var image: UIImage?

        image = info[.originalImage] as? UIImage

        let ciImage = CIImage(image: image!)!

        let context = CIContext(options: nil)
        let detector = CIDetector(ofType: CIDetectorTypeQRCode, context: context, options: [CIDetectorAccuracy: CIDetectorAccuracyHigh])

        let features = detector?.features(in: ciImage)
        for feature in features as! [CIQRCodeFeature] {
            qrString = feature.messageString ?? ""
            print("图片读取到的二维码", feature.messageString ?? "")
            if let b = resultBlock {
                b(qrString)
            }
            dismiss(animated: true) {}
        }
    }
}
